@@ -18,9 +18,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -52,9 +50,6 @@ meta_window_ensure_frame (MetaWindow *window)
   if (window->frame)
     return;
   
-  /* See comment below for why this is required. */
-  meta_display_grab (window->display);
-  
   frame = g_new (MetaFrame, 1);
 
   frame->window = window;
@@ -120,14 +115,6 @@ meta_window_ensure_frame (MetaWindow *window)
   
   meta_display_register_x_window (window->display, &frame->xwindow, window);
 
-  /* Reparent the client window; it may be destroyed,
-   * thus the error trap. We'll get a destroy notify later
-   * and free everything. Comment in FVWM source code says
-   * we need a server grab or the child can get its MapNotify
-   * before we've finished reparenting and getting the decoration
-   * window onscreen, so ensure_frame must be called with
-   * a grab.
-   */
   meta_error_trap_push (window->display);
   if (window->mapped)
     {
@@ -170,8 +157,6 @@ meta_window_ensure_frame (MetaWindow *window)
 
   /* Move keybindings to frame instead of window */
   meta_window_grab_keys (window);
-
-  meta_display_ungrab (window->display);
 }
 
 void
