@@ -14,9 +14,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Giovanni Campagna <gcampagn@redhat.com>
  */
@@ -307,7 +305,6 @@ meta_cursor_reference_from_theme (MetaCursorTracker  *tracker,
   self->texture = cogl_texture_2d_new_from_data (cogl_context,
                                                  width, height,
                                                  cogl_format,
-                                                 COGL_PIXEL_FORMAT_ANY,
                                                  rowstride,
                                                  (uint8_t*)image->pixels,
                                                  NULL);
@@ -367,7 +364,7 @@ meta_cursor_reference_from_buffer (MetaCursorTracker  *tracker,
   ClutterBackend *backend;
   CoglContext *cogl_context;
   MetaCursorReference *self;
-  CoglPixelFormat cogl_format, cogl_internal_format;
+  CoglPixelFormat cogl_format;
   struct wl_shm_buffer *shm_buffer;
   uint32_t gbm_format;
 
@@ -391,37 +388,31 @@ meta_cursor_reference_from_buffer (MetaCursorTracker  *tracker,
 #if G_BYTE_ORDER == G_BIG_ENDIAN
           case WL_SHM_FORMAT_ARGB8888:
             cogl_format = COGL_PIXEL_FORMAT_ARGB_8888_PRE;
-            cogl_internal_format = COGL_PIXEL_FORMAT_ANY;
             gbm_format = GBM_FORMAT_ARGB8888;
             break;
           case WL_SHM_FORMAT_XRGB8888:
             cogl_format = COGL_PIXEL_FORMAT_ARGB_8888;
-            cogl_internal_format = COGL_PIXEL_FORMAT_RGB_888;
             gbm_format = GBM_FORMAT_XRGB8888;
             break;
 #else
           case WL_SHM_FORMAT_ARGB8888:
             cogl_format = COGL_PIXEL_FORMAT_BGRA_8888_PRE;
-            cogl_internal_format = COGL_PIXEL_FORMAT_ANY;
             gbm_format = GBM_FORMAT_ARGB8888;
             break;
           case WL_SHM_FORMAT_XRGB8888:
             cogl_format = COGL_PIXEL_FORMAT_BGRA_8888;
-            cogl_internal_format = COGL_PIXEL_FORMAT_BGR_888;
             gbm_format = GBM_FORMAT_XRGB8888;
             break;
 #endif
           default:
             g_warn_if_reached ();
             cogl_format = COGL_PIXEL_FORMAT_ARGB_8888;
-            cogl_internal_format = COGL_PIXEL_FORMAT_ANY;
             gbm_format = GBM_FORMAT_ARGB8888;
         }
 
       self->texture = cogl_texture_2d_new_from_data (cogl_context,
                                                      width, height,
                                                      cogl_format,
-                                                     cogl_internal_format,
                                                      rowstride,
                                                      wl_shm_buffer_get_data (shm_buffer),
                                                      NULL);
@@ -477,47 +468,10 @@ meta_cursor_reference_from_buffer (MetaCursorTracker  *tracker,
 
       if (tracker->gbm)
         {
-          cogl_format = cogl_texture_get_format (COGL_TEXTURE (self->texture));
-          switch (cogl_format)
-            {
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
-            case COGL_PIXEL_FORMAT_ARGB_8888_PRE:
-            case COGL_PIXEL_FORMAT_ARGB_8888:
-              gbm_format = GBM_FORMAT_BGRA8888;
-              break;
-            case COGL_PIXEL_FORMAT_BGRA_8888_PRE:
-            case COGL_PIXEL_FORMAT_BGRA_8888:
-              break;
-            case COGL_PIXEL_FORMAT_RGB_888:
-              break;
-#else
-            case COGL_PIXEL_FORMAT_ARGB_8888_PRE:
-            case COGL_PIXEL_FORMAT_ARGB_8888:
-              gbm_format = GBM_FORMAT_ARGB8888;
-              break;
-            case COGL_PIXEL_FORMAT_BGRA_8888_PRE:
-            case COGL_PIXEL_FORMAT_BGRA_8888:
-              gbm_format = GBM_FORMAT_BGRA8888;
-              break;
-            case COGL_PIXEL_FORMAT_RGB_888:
-              gbm_format = GBM_FORMAT_RGB888;
-              break;
-#endif
-            default:
-              meta_warning ("Unknown cogl format %d\n", cogl_format);
-              return self;
-            }
-
-          if (gbm_device_is_format_supported (tracker->gbm, gbm_format,
-                                              GBM_BO_USE_CURSOR_64X64))
-            {
-              self->bo = gbm_bo_import (tracker->gbm, GBM_BO_IMPORT_WL_BUFFER,
-                                        buffer, GBM_BO_USE_CURSOR_64X64);
-              if (!self->bo)
-                meta_warning ("Importing HW cursor from wl_buffer failed\n");
-            }
-          else
-            meta_warning ("HW cursor for format %d not supported\n", gbm_format);
+          self->bo = gbm_bo_import (tracker->gbm, GBM_BO_IMPORT_WL_BUFFER,
+                                    buffer, GBM_BO_USE_CURSOR_64X64);
+          if (!self->bo)
+            meta_warning ("Importing HW cursor from wl_buffer failed\n");
         }
      }
 
@@ -616,9 +570,12 @@ make_wayland_cursor_tracker (MetaScreen *screen)
   compositor = meta_wayland_compositor_get_default ();
   compositor->seat->cursor_tracker = self;
 
-  self->drm_fd = compositor->drm_fd;
-  if (self->drm_fd >= 0)
-    self->gbm = gbm_create_device (compositor->drm_fd);
+  if (meta_wayland_compositor_is_native (compositor))
+    {
+      CoglRenderer *cogl_renderer = cogl_display_get_renderer (cogl_context_get_display (ctx));
+      self->drm_fd = cogl_kms_renderer_get_kms_fd (cogl_renderer);
+      self->gbm = gbm_create_device (self->drm_fd);
+    }
 
   monitors = meta_monitor_manager_get ();
   g_signal_connect_object (monitors, "monitors-changed",
@@ -747,7 +704,6 @@ ensure_xfixes_cursor (MetaCursorTracker *tracker)
                                           cursor_image->width,
                                           cursor_image->height,
                                           CLUTTER_CAIRO_FORMAT_ARGB32,
-                                          COGL_PIXEL_FORMAT_ANY,
                                           cursor_image->width * 4, /* stride */
                                           cursor_data,
                                           NULL);
