@@ -58,6 +58,52 @@
 #include "meta-idle-monitor-private.h"
 #include "monitor-private.h"
 
+typedef enum
+{
+  META_WAYLAND_SUBSURFACE_PLACEMENT_ABOVE,
+  META_WAYLAND_SUBSURFACE_PLACEMENT_BELOW
+} MetaWaylandSubsurfacePlacement;
+
+typedef struct
+{
+  MetaWaylandSubsurfacePlacement placement;
+  MetaWaylandSurface *sibling;
+  struct wl_listener sibling_destroy_listener;
+} MetaWaylandSubsurfacePlacementOp;
+
+static void
+surface_handle_buffer_destroy (struct wl_listener *listener, void *data)
+{
+  MetaWaylandSurface *surface = wl_container_of (listener, surface, buffer_destroy_listener);
+
+  wl_resource_post_error (surface->resource, WL_DISPLAY_ERROR_INVALID_OBJECT,
+                          "Destroyed buffer while it was attached to the surface");
+  surface->buffer = NULL;
+  wl_list_remove (&surface->buffer_destroy_listener.link);
+}
+
+static void
+surface_set_buffer (MetaWaylandSurface *surface,
+                    MetaWaylandBuffer  *buffer)
+{
+  if (surface->buffer == buffer)
+    return;
+
+  if (surface->buffer)
+    {
+      meta_wayland_buffer_unref (surface->buffer);
+      wl_list_remove (&surface->buffer_destroy_listener.link);
+    }
+
+  surface->buffer = buffer;
+
+  if (surface->buffer)
+    {
+      meta_wayland_buffer_ref (surface->buffer);
+      wl_signal_add (&surface->buffer->destroy_signal, &surface->buffer_destroy_listener);
+    }
+}
+
 static void
 surface_process_damage (MetaWaylandSurface *surface,
                         cairo_region_t *region)
@@ -78,28 +124,28 @@ surface_process_damage (MetaWaylandSurface *surface,
 }
 
 static void
-meta_wayland_surface_destroy (struct wl_client *wayland_client,
-                              struct wl_resource *wayland_resource)
+meta_wayland_surface_destroy (struct wl_client *client,
+                              struct wl_resource *resource)
 {
-  wl_resource_destroy (wayland_resource);
+  wl_resource_destroy (resource);
 }
 
 static void
-meta_wayland_surface_attach (struct wl_client *wayland_client,
-                             struct wl_resource *wayland_surface_resource,
-                             struct wl_resource *wayland_buffer_resource,
+meta_wayland_surface_attach (struct wl_client *client,
+                             struct wl_resource *surface_resource,
+                             struct wl_resource *buffer_resource,
                              gint32 dx, gint32 dy)
 {
   MetaWaylandSurface *surface =
-    wl_resource_get_user_data (wayland_surface_resource);
+    wl_resource_get_user_data (surface_resource);
   MetaWaylandBuffer *buffer;
 
   /* X11 unmanaged window */
   if (!surface)
     return;
 
-  if (wayland_buffer_resource)
-    buffer = meta_wayland_buffer_from_resource (wayland_buffer_resource);
+  if (buffer_resource)
+    buffer = meta_wayland_buffer_from_resource (buffer_resource);
   else
     buffer = NULL;
 
@@ -159,11 +205,8 @@ meta_wayland_surface_frame (struct wl_client *client,
 
   callback = g_slice_new0 (MetaWaylandFrameCallback);
   callback->compositor = surface->compositor;
-  callback->resource = wl_resource_create (client,
-					   &wl_callback_interface, 1,
-					   callback_id);
-  wl_resource_set_user_data (callback->resource, callback);
-  wl_resource_set_destructor (callback->resource, destroy_frame_callback);
+  callback->resource = wl_resource_create (client, &wl_callback_interface, META_WL_CALLBACK_VERSION, callback_id);
+  wl_resource_set_implementation (callback->resource, NULL, callback, destroy_frame_callback);
 
   wl_list_insert (surface->pending.frame_callback_list.prev, &callback->link);
 }
@@ -239,52 +282,47 @@ ensure_buffer_texture (MetaWaylandBuffer *buffer)
 }
 
 static void
-cursor_surface_commit (MetaWaylandSurface *surface)
+cursor_surface_commit (MetaWaylandSurface             *surface,
+                       MetaWaylandDoubleBufferedState *pending,
+                       gboolean                        buffer_changed)
 {
-  MetaWaylandBuffer *buffer = surface->pending.buffer;
-
-  if (surface->pending.newly_attached && buffer != surface->buffer_ref.buffer)
-    {
-      ensure_buffer_texture (buffer);
-      meta_wayland_buffer_reference (&surface->buffer_ref, buffer);
-    }
-
-  meta_wayland_seat_update_sprite (surface->compositor->seat);
-}
-
-static gboolean
-actor_surface_commit (MetaWaylandSurface *surface)
-{
-  MetaSurfaceActor *surface_actor = surface->surface_actor;
-  MetaWaylandBuffer *buffer = surface->pending.buffer;
-  gboolean changed = FALSE;
-
-  /* wl_surface.attach */
-  if (surface->pending.newly_attached && buffer != surface->buffer_ref.buffer)
-    {
-      ensure_buffer_texture (buffer);
-      meta_wayland_buffer_reference (&surface->buffer_ref, buffer);
-      meta_surface_actor_attach_wayland_buffer (surface_actor, buffer);
-      changed = TRUE;
-    }
-
-  surface_process_damage (surface, surface->pending.damage);
-
-  if (surface->pending.opaque_region)
-    meta_surface_actor_set_opaque_region (surface_actor, surface->pending.opaque_region);
-  if (surface->pending.input_region)
-    meta_surface_actor_set_input_region (surface_actor, surface->pending.input_region);
-
-  return changed;
+  if (buffer_changed)
+    meta_wayland_seat_update_cursor_surface (surface->compositor->seat);
 }
 
 static void
-toplevel_surface_commit (MetaWaylandSurface *surface)
+actor_surface_commit (MetaWaylandSurface             *surface,
+                      MetaWaylandDoubleBufferedState *pending,
+                      gboolean                        buffer_changed)
 {
-  if (actor_surface_commit (surface))
+  MetaSurfaceActor *surface_actor = surface->surface_actor;
+  MetaWaylandBuffer *buffer = pending->buffer;
+
+  if (buffer_changed)
+    {
+      ensure_buffer_texture (buffer);
+      meta_surface_actor_attach_wayland_buffer (surface_actor, buffer);
+    }
+
+  surface_process_damage (surface, pending->damage);
+
+  if (pending->opaque_region)
+    meta_surface_actor_set_opaque_region (surface_actor, pending->opaque_region);
+  if (pending->input_region)
+    meta_surface_actor_set_input_region (surface_actor, pending->input_region);
+}
+
+static void
+toplevel_surface_commit (MetaWaylandSurface             *surface,
+                         MetaWaylandDoubleBufferedState *pending,
+                         gboolean                        buffer_changed)
+{
+  actor_surface_commit (surface, pending, buffer_changed);
+
+  if (buffer_changed)
     {
       MetaWindow *window = surface->window;
-      MetaWaylandBuffer *buffer = surface->pending.buffer;
+      MetaWaylandBuffer *buffer = pending->buffer;
 
       meta_window_set_surface_mapped (window, buffer != NULL);
       /* We resize X based surfaces according to X events */
@@ -293,37 +331,185 @@ toplevel_surface_commit (MetaWaylandSurface *surface)
           int new_width;
           int new_height;
 
-          new_width = surface->buffer_ref.buffer->width;
-          new_height = surface->buffer_ref.buffer->height;
+          new_width = surface->buffer->width;
+          new_height = surface->buffer->height;
           if (new_width != window->rect.width ||
               new_height != window->rect.height ||
-              surface->pending.dx != 0 ||
-              surface->pending.dy != 0)
-            meta_window_move_resize_wayland (window, new_width, new_height,
-                                             surface->pending.dx, surface->pending.dy);
+              pending->dx != 0 ||
+              pending->dy != 0)
+            meta_window_move_resize_wayland (window, new_width, new_height, pending->dx, pending->dy);
         }
     }
 }
 
 static void
-subsurface_surface_commit (MetaWaylandSurface *surface)
+surface_handle_pending_buffer_destroy (struct wl_listener *listener, void *data)
 {
-  if (actor_surface_commit (surface))
+  MetaWaylandDoubleBufferedState *state =
+    wl_container_of (listener, state, buffer_destroy_listener);
+
+  state->buffer = NULL;
+}
+
+static void
+double_buffered_state_init (MetaWaylandDoubleBufferedState *state)
+{
+  state->newly_attached = FALSE;
+  state->buffer = NULL;
+  state->dx = 0;
+  state->dy = 0;
+
+  state->damage = cairo_region_create ();
+  state->buffer_destroy_listener.notify =
+    surface_handle_pending_buffer_destroy;
+  wl_list_init (&state->frame_callback_list);
+}
+
+static void
+double_buffered_state_destroy (MetaWaylandDoubleBufferedState *state)
+{
+  MetaWaylandFrameCallback *cb, *next;
+
+  g_clear_pointer (&state->damage, cairo_region_destroy);
+  g_clear_pointer (&state->input_region, cairo_region_destroy);
+  g_clear_pointer (&state->opaque_region, cairo_region_destroy);
+
+  if (state->buffer)
+    wl_list_remove (&state->buffer_destroy_listener.link);
+  wl_list_for_each_safe (cb, next, &state->frame_callback_list, link)
+    wl_resource_destroy (cb->resource);
+}
+
+static void
+double_buffered_state_reset (MetaWaylandDoubleBufferedState *state)
+{
+  double_buffered_state_destroy (state);
+  double_buffered_state_init (state);
+}
+
+static void
+move_double_buffered_state (MetaWaylandDoubleBufferedState *from,
+                            MetaWaylandDoubleBufferedState *to)
+{
+  if (from->buffer)
+    wl_list_remove (&from->buffer_destroy_listener.link);
+
+  to->newly_attached = from->newly_attached;
+  from->newly_attached = FALSE;
+
+  to->buffer = from->buffer;
+  from->buffer = NULL;
+  if (to->buffer)
+    wl_signal_add (&to->buffer->destroy_signal, &to->buffer_destroy_listener);
+
+  to->dx = from->dx;
+  to->dy = from->dy;
+  from->dx = from->dy = 0;
+
+  empty_region (to->damage);
+  cairo_region_union (to->damage, from->damage);
+  empty_region (from->damage);
+
+  g_clear_pointer (&to->input_region, cairo_region_destroy);
+  g_clear_pointer (&to->opaque_region, cairo_region_destroy);
+  to->input_region = from->input_region;
+  to->opaque_region = from->opaque_region;
+  from->input_region = from->opaque_region = NULL;
+
+  wl_list_init (&to->frame_callback_list);
+  wl_list_insert_list (&to->frame_callback_list, &from->frame_callback_list);
+  wl_list_init (&from->frame_callback_list);
+}
+
+static void
+subsurface_surface_commit (MetaWaylandSurface             *surface,
+                           MetaWaylandDoubleBufferedState *pending,
+                           gboolean                        buffer_changed)
+{
+  /*
+   * If the sub-surface is in synchronous mode, post-pone the commit of its
+   * state until the sub-surface parent commits.
+   *
+   * This is done by moving the various states (damage, input region, buffer
+   * etc.) from the buffered state pending commit to the sub-surface's pending
+   * buffered state.
+   *
+   * The sub-surface's pending buffered state will be committed to the
+   * associated surface when its parent surface is committed, or if the user
+   * issues a wl_subsurface.set_desync request.
+   */
+  if (surface->sub.synchronous)
     {
-      MetaSurfaceActor *surface_actor = surface->surface_actor;
-      MetaWaylandBuffer *buffer = surface->pending.buffer;
-      float x, y;
-
-      if (buffer != NULL)
-        clutter_actor_show (CLUTTER_ACTOR (surface_actor));
-      else
-        clutter_actor_hide (CLUTTER_ACTOR (surface_actor));
-
-      clutter_actor_get_position (CLUTTER_ACTOR (surface_actor), &x, &y);
-      x += surface->pending.dx;
-      y += surface->pending.dy;
-      clutter_actor_set_position (CLUTTER_ACTOR (surface_actor), x, y);
+      move_double_buffered_state (pending, &surface->sub.pending_surface_state);
     }
+  else
+    {
+      actor_surface_commit (surface, pending, buffer_changed);
+
+      if (buffer_changed)
+        {
+          MetaSurfaceActor *surface_actor = surface->surface_actor;
+          MetaWaylandBuffer *buffer = pending->buffer;
+          float x, y;
+
+          if (buffer != NULL)
+            clutter_actor_show (CLUTTER_ACTOR (surface_actor));
+          else
+            clutter_actor_hide (CLUTTER_ACTOR (surface_actor));
+
+          clutter_actor_get_position (CLUTTER_ACTOR (surface_actor), &x, &y);
+          x += pending->dx;
+          y += pending->dy;
+          clutter_actor_set_position (CLUTTER_ACTOR (surface_actor), x, y);
+        }
+    }
+}
+
+static void
+subsurface_parent_surface_committed (MetaWaylandSurface *surface);
+
+static void
+parent_surface_committed (gpointer data, gpointer user_data)
+{
+  subsurface_parent_surface_committed (data);
+}
+
+static void
+commit_double_buffered_state (MetaWaylandSurface             *surface,
+                              MetaWaylandDoubleBufferedState *pending)
+{
+  MetaWaylandCompositor *compositor = surface->compositor;
+  gboolean buffer_changed = FALSE;
+
+  /* wl_surface.attach */
+  if (pending->newly_attached && surface->buffer != pending->buffer)
+    {
+      surface_set_buffer (surface, pending->buffer);
+      buffer_changed = TRUE;
+    }
+
+  if (surface == compositor->seat->cursor_surface)
+    cursor_surface_commit (surface, pending, buffer_changed);
+  else if (surface->window)
+    toplevel_surface_commit (surface, pending, buffer_changed);
+  else if (surface->subsurface.resource)
+    subsurface_surface_commit (surface, pending, buffer_changed);
+
+  g_list_foreach (surface->subsurfaces,
+                  parent_surface_committed,
+                  NULL);
+
+  if (pending->buffer)
+    {
+      wl_list_remove (&pending->buffer_destroy_listener.link);
+      pending->buffer = NULL;
+    }
+
+  /* wl_surface.frame */
+  wl_list_insert_list (&compositor->frame_callbacks, &pending->frame_callback_list);
+  wl_list_init (&pending->frame_callback_list);
+
+  double_buffered_state_reset (pending);
 }
 
 static void
@@ -331,38 +517,12 @@ meta_wayland_surface_commit (struct wl_client *client,
                              struct wl_resource *resource)
 {
   MetaWaylandSurface *surface = wl_resource_get_user_data (resource);
-  MetaWaylandCompositor *compositor;
 
   /* X11 unmanaged window */
   if (!surface)
     return;
 
-  compositor = surface->compositor;
-
-  if (surface == compositor->seat->sprite)
-    cursor_surface_commit (surface);
-  else if (surface->window)
-    toplevel_surface_commit (surface);
-  else if (surface->subsurface.resource)
-    subsurface_surface_commit (surface);
-
-  if (surface->pending.buffer)
-    {
-      wl_list_remove (&surface->pending.buffer_destroy_listener.link);
-      surface->pending.buffer = NULL;
-    }
-
-  surface->pending.dx = 0;
-  surface->pending.dy = 0;
-  surface->pending.newly_attached = FALSE;
-  g_clear_pointer (&surface->pending.opaque_region, cairo_region_destroy);
-  g_clear_pointer (&surface->pending.input_region, cairo_region_destroy);
-  empty_region (surface->pending.damage);
-
-  /* wl_surface.frame */
-  wl_list_insert_list (&compositor->frame_callbacks,
-                       &surface->pending.frame_callback_list);
-  wl_list_init (&surface->pending.frame_callback_list);
+  commit_double_buffered_state (surface, &surface->pending);
 }
 
 static void
@@ -398,28 +558,17 @@ static void
 meta_wayland_surface_free (MetaWaylandSurface *surface)
 {
   MetaWaylandCompositor *compositor = surface->compositor;
-  MetaWaylandFrameCallback *cb, *next;
 
   compositor->surfaces = g_list_remove (compositor->surfaces, surface);
 
-  meta_wayland_buffer_reference (&surface->buffer_ref, NULL);
-
-  if (surface->pending.buffer)
-    wl_list_remove (&surface->pending.buffer_destroy_listener.link);
-
-  cairo_region_destroy (surface->pending.damage);
-
-  wl_list_for_each_safe (cb, next,
-                         &surface->pending.frame_callback_list, link)
-    wl_resource_destroy (cb->resource);
-
-  meta_wayland_compositor_repick (compositor);
-
+  surface_set_buffer (surface, NULL);
+  double_buffered_state_destroy (&surface->pending);
   g_object_unref (surface->surface_actor);
-
   if (surface->resource)
     wl_resource_set_user_data (surface->resource, NULL);
   g_slice_free (MetaWaylandSurface, surface);
+
+  meta_wayland_compositor_repick (compositor);
 }
 
 static void
@@ -473,19 +622,9 @@ meta_wayland_surface_resource_destroy_cb (struct wl_resource *resource)
     }
 }
 
-static void
-surface_handle_pending_buffer_destroy (struct wl_listener *listener,
-                                       void *data)
-{
-  MetaWaylandSurface *surface =
-    wl_container_of (listener, surface, pending.buffer_destroy_listener);
-
-  surface->pending.buffer = NULL;
-}
-
 MetaWaylandSurface *
 meta_wayland_surface_create (MetaWaylandCompositor *compositor,
-			     struct wl_client      *wayland_client,
+			     struct wl_client      *client,
 			     guint32                id,
 			     guint32                version)
 {
@@ -493,17 +632,13 @@ meta_wayland_surface_create (MetaWaylandCompositor *compositor,
 
   surface->compositor = compositor;
 
-  surface->resource = wl_resource_create (wayland_client,
-					  &wl_surface_interface,
-					  version, id);
+  surface->resource = wl_resource_create (client, &wl_surface_interface, version, id);
   wl_resource_set_implementation (surface->resource, &meta_wayland_surface_interface, surface,
 				  meta_wayland_surface_resource_destroy_cb);
 
-  surface->pending.damage = cairo_region_create ();
+  double_buffered_state_init (&surface->pending);
 
-  surface->pending.buffer_destroy_listener.notify =
-    surface_handle_pending_buffer_destroy;
-  wl_list_init (&surface->pending.frame_callback_list);
+  surface->buffer_destroy_listener.notify = surface_handle_buffer_destroy;
 
   surface->surface_actor = g_object_ref_sink (meta_surface_actor_new ());
   return surface;
@@ -513,7 +648,6 @@ static void
 destroy_surface_extension (MetaWaylandSurfaceExtension *extension)
 {
   wl_list_remove (&extension->surface_destroy_listener.link);
-  extension->surface_destroy_listener.notify = NULL;
   extension->resource = NULL;
 }
 
@@ -543,17 +677,15 @@ create_surface_extension (MetaWaylandSurfaceExtension *extension,
                           const void                  *implementation,
                           wl_resource_destroy_func_t   destructor)
 {
-  struct wl_resource *resource;
-
   if (extension->resource != NULL)
     return FALSE;
 
-  resource = wl_resource_create (client, interface, get_resource_version (master_resource, max_version), id);
-  wl_resource_set_implementation (resource, implementation, extension, destructor);
+  extension->resource = wl_resource_create (client, interface, get_resource_version (master_resource, max_version), id);
+  wl_resource_set_implementation (extension->resource, implementation, extension, destructor);
 
-  extension->resource = resource;
   extension->surface_destroy_listener.notify = extension_handle_surface_destroy;
   wl_resource_add_destroy_listener (surface_resource, &extension->surface_destroy_listener);
+
   return TRUE;
 }
 
@@ -815,7 +947,7 @@ get_xdg_surface (struct wl_client *client,
       return;
     }
 
-  surface->window = meta_window_new_for_wayland (meta_get_display (), surface);
+  surface->window = meta_window_wayland_new (meta_get_display (), surface);
 }
 
 static void
@@ -885,7 +1017,7 @@ get_xdg_popup (struct wl_client *client,
       return;
     }
 
-  surface->window = meta_window_new_for_wayland (meta_get_display (), surface);
+  surface->window = meta_window_wayland_new (meta_get_display (), surface);
   surface->window->rect.x = parent_rect.x + x;
   surface->window->rect.y = parent_rect.y + y;
   surface->window->showing_for_first_time = FALSE;
@@ -993,13 +1125,79 @@ bind_gtk_shell (struct wl_client *client,
   gtk_shell_send_capabilities (resource, GTK_SHELL_CAPABILITY_GLOBAL_APP_MENU);
 }
 
+static void
+subsurface_parent_surface_committed (MetaWaylandSurface *surface)
+{
+  MetaWaylandDoubleBufferedState *pending_surface_state = &surface->sub.pending_surface_state;
+
+  if (surface->sub.pending_pos)
+    {
+      clutter_actor_set_position (CLUTTER_ACTOR (surface->surface_actor),
+                                  surface->sub.pending_x,
+                                  surface->sub.pending_y);
+      surface->sub.pending_pos = FALSE;
+    }
+
+  if (surface->sub.pending_placement_ops)
+    {
+      GSList *it;
+      for (it = surface->sub.pending_placement_ops; it; it = it->next)
+        {
+          MetaWaylandSubsurfacePlacementOp *op = it->data;
+          ClutterActor *surface_actor;
+          ClutterActor *parent_actor;
+          ClutterActor *sibling_actor;
+
+          if (!op->sibling)
+            {
+              g_slice_free (MetaWaylandSubsurfacePlacementOp, op);
+              continue;
+            }
+
+          surface_actor = CLUTTER_ACTOR (surface->surface_actor);
+          parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->sub.parent));
+          sibling_actor = CLUTTER_ACTOR (op->sibling->surface_actor);
+
+          switch (op->placement)
+            {
+            case META_WAYLAND_SUBSURFACE_PLACEMENT_ABOVE:
+              clutter_actor_set_child_above_sibling (parent_actor, surface_actor, sibling_actor);
+              break;
+            case META_WAYLAND_SUBSURFACE_PLACEMENT_BELOW:
+              clutter_actor_set_child_below_sibling (parent_actor, surface_actor, sibling_actor);
+              break;
+            }
+
+          wl_list_remove (&op->sibling_destroy_listener.link);
+          g_slice_free (MetaWaylandSubsurfacePlacementOp, op);
+        }
+
+      g_slist_free (surface->sub.pending_placement_ops);
+      surface->sub.pending_placement_ops = NULL;
+    }
+
+  if (surface->sub.synchronous)
+    commit_double_buffered_state (surface, pending_surface_state);
+
+  double_buffered_state_reset (pending_surface_state);
+}
+
 static void
 wl_subsurface_destructor (struct wl_resource *resource)
 {
   MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
 
-  unparent_actor (surface);
+  if (surface->sub.parent)
+    {
+      wl_list_remove (&surface->sub.parent_destroy_listener.link);
+      surface->sub.parent->subsurfaces =
+        g_list_remove (surface->sub.parent->subsurfaces, surface);
+      unparent_actor (surface);
+      surface->sub.parent = NULL;
+    }
+
+  double_buffered_state_destroy (&surface->sub.pending_surface_state);
   destroy_surface_extension (subsurface);
 }
 
@@ -1019,7 +1217,47 @@ wl_subsurface_set_position (struct wl_client *client,
   MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
 
-  clutter_actor_set_position (CLUTTER_ACTOR (surface->surface_actor), x, y);
+  surface->sub.pending_x = x;
+  surface->sub.pending_y = y;
+  surface->sub.pending_pos = TRUE;
+}
+
+static gboolean
+is_valid_sibling (MetaWaylandSurface *surface, MetaWaylandSurface *sibling)
+{
+  if (surface->sub.parent == sibling)
+    return TRUE;
+  if (surface->sub.parent == sibling->sub.parent)
+    return TRUE;
+  return FALSE;
+}
+
+static void
+subsurface_handle_pending_sibling_destroyed (struct wl_listener *listener, void *data)
+{
+  MetaWaylandSubsurfacePlacementOp *op =
+    wl_container_of (listener, op, sibling_destroy_listener);
+
+  op->sibling = NULL;
+}
+
+static void
+queue_subsurface_placement (MetaWaylandSurface *surface,
+                            MetaWaylandSurface *sibling,
+                            MetaWaylandSubsurfacePlacement placement)
+{
+  MetaWaylandSubsurfacePlacementOp *op =
+    g_slice_new (MetaWaylandSubsurfacePlacementOp);
+
+  op->placement = placement;
+  op->sibling = sibling;
+  op->sibling_destroy_listener.notify =
+    subsurface_handle_pending_sibling_destroyed;
+  wl_resource_add_destroy_listener (sibling->resource,
+                                    &op->sibling_destroy_listener);
+
+  surface->sub.pending_placement_ops =
+    g_slist_append (surface->sub.pending_placement_ops, op);
 }
 
 static void
@@ -1027,16 +1265,22 @@ wl_subsurface_place_above (struct wl_client *client,
                            struct wl_resource *resource,
                            struct wl_resource *sibling_resource)
 {
-  ClutterActor *parent_actor;
   MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
   MetaWaylandSurface *sibling = wl_resource_get_user_data (sibling_resource);
 
-  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
+  if (!is_valid_sibling (surface, sibling))
+    {
+      wl_resource_post_error (resource, WL_SUBSURFACE_ERROR_BAD_SURFACE,
+                              "wl_subsurface::place_above: wl_surface@%d is "
+                              "not a valid parent or sibling",
+                              wl_resource_get_id (sibling->resource));
+      return;
+    }
 
-  clutter_actor_set_child_above_sibling (parent_actor,
-                                         CLUTTER_ACTOR (surface->surface_actor),
-                                         CLUTTER_ACTOR (sibling->surface_actor));
+  queue_subsurface_placement (surface,
+                              sibling,
+                              META_WAYLAND_SUBSURFACE_PLACEMENT_ABOVE);
 }
 
 static void
@@ -1044,30 +1288,49 @@ wl_subsurface_place_below (struct wl_client *client,
                            struct wl_resource *resource,
                            struct wl_resource *sibling_resource)
 {
-  ClutterActor *parent_actor;
   MetaWaylandSurfaceExtension *subsurface = wl_resource_get_user_data (resource);
   MetaWaylandSurface *surface = wl_container_of (subsurface, surface, subsurface);
   MetaWaylandSurface *sibling = wl_resource_get_user_data (sibling_resource);
 
-  parent_actor = clutter_actor_get_parent (CLUTTER_ACTOR (surface->surface_actor));
+  if (!is_valid_sibling (surface, sibling))
+    {
+      wl_resource_post_error (resource, WL_SUBSURFACE_ERROR_BAD_SURFACE,
+                              "wl_subsurface::place_below: wl_surface@%d is "
+                              "not a valid parent or sibling",
+                              wl_resource_get_id (sibling->resource));
+      return;
+    }
 
-  clutter_actor_set_child_below_sibling (parent_actor,
-                                         CLUTTER_ACTOR (surface->surface_actor),
-                                         CLUTTER_ACTOR (sibling->surface_actor));
+  queue_subsurface_placement (surface,
+                              sibling,
+                              META_WAYLAND_SUBSURFACE_PLACEMENT_BELOW);
 }
 
 static void
 wl_subsurface_set_sync (struct wl_client *client,
                         struct wl_resource *resource)
 {
-  g_warning ("TODO: support wl_subsurface.set_sync");
+  MetaWaylandSurfaceExtension *subsurface =
+    wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface =
+    wl_container_of (subsurface, surface, subsurface);
+
+  surface->sub.synchronous = TRUE;
 }
 
 static void
 wl_subsurface_set_desync (struct wl_client *client,
                           struct wl_resource *resource)
 {
-  g_warning ("TODO: support wl_subsurface.set_desync");
+  MetaWaylandSurfaceExtension *subsurface =
+    wl_resource_get_user_data (resource);
+  MetaWaylandSurface *surface =
+    wl_container_of (subsurface, surface, subsurface);
+
+  if (surface->sub.synchronous)
+    subsurface_parent_surface_committed (surface);
+
+  surface->sub.synchronous = FALSE;
 }
 
 static const struct wl_subsurface_interface meta_wayland_subsurface_interface = {
@@ -1086,6 +1349,18 @@ wl_subcompositor_destroy (struct wl_client *client,
   wl_resource_destroy (resource);
 }
 
+static void
+surface_handle_parent_surface_destroyed (struct wl_listener *listener,
+                                         void *data)
+{
+  MetaWaylandSurface *surface = wl_container_of (listener,
+                                                 surface,
+                                                 sub.parent_destroy_listener);
+
+  surface->sub.parent = NULL;
+  unparent_actor (surface);
+}
+
 static void
 wl_subcompositor_get_subsurface (struct wl_client *client,
                                  struct wl_resource *resource,
@@ -1108,6 +1383,14 @@ wl_subcompositor_get_subsurface (struct wl_client *client,
       return;
     }
 
+  double_buffered_state_init (&surface->sub.pending_surface_state);
+  surface->sub.parent = parent;
+  surface->sub.parent_destroy_listener.notify =
+    surface_handle_parent_surface_destroyed;
+  wl_resource_add_destroy_listener (parent->resource,
+                                    &surface->sub.parent_destroy_listener);
+  parent->subsurfaces = g_list_append (parent->subsurfaces, surface);
+
   clutter_actor_add_child (CLUTTER_ACTOR (parent->surface_actor),
                            CLUTTER_ACTOR (surface->surface_actor));
 }
