@@ -39,13 +39,7 @@ struct _MetaWaylandBuffer
 
   CoglTexture *texture;
   int32_t width, height;
-  uint32_t busy_count;
-};
-
-struct _MetaWaylandBufferReference
-{
-  MetaWaylandBuffer *buffer;
-  struct wl_listener destroy_listener;
+  uint32_t ref_count;
 };
 
 typedef struct
@@ -77,7 +71,6 @@ struct _MetaWaylandSurface
 {
   struct wl_resource *resource;
   MetaWaylandCompositor *compositor;
-  MetaWaylandBufferReference buffer_ref;
   MetaSurfaceActor *surface_actor;
   MetaWindow *window;
   MetaWaylandSurfaceExtension xdg_surface;
@@ -85,6 +78,24 @@ struct _MetaWaylandSurface
   MetaWaylandSurfaceExtension gtk_surface;
   MetaWaylandSurfaceExtension subsurface;
 
+  MetaWaylandBuffer *buffer;
+  struct wl_listener buffer_destroy_listener;
+
+  GList *subsurfaces;
+
+  struct {
+    MetaWaylandSurface *parent;
+    struct wl_listener parent_destroy_listener;
+
+    gboolean synchronous;
+    MetaWaylandDoubleBufferedState pending_surface_state;
+
+    int32_t pending_x;
+    int32_t pending_y;
+    gboolean pending_pos;
+    GSList *pending_placement_ops;
+  } sub;
+
   /* All the pending state, that wl_surface.commit will apply. */
   MetaWaylandDoubleBufferedState pending;
 };
