@@ -44,15 +44,17 @@ xserver_set_window_id (struct wl_client *client,
   MetaWindow *window;
 
   window = meta_display_lookup_x_window (display, xid);
-  if (window)
-    {
-      surface->window = window;
-      window->surface = surface;
-    }
+  if (!window)
+    return;
+
+  surface->window = window;
+  window->surface = surface;
+
+  meta_window_set_surface_mapped (window, TRUE);
 }
 
 static const struct xserver_interface xserver_implementation = {
-    xserver_set_window_id
+  xserver_set_window_id
 };
 
 static void
@@ -73,13 +75,8 @@ bind_xserver (struct wl_client *client,
   wl_resource_set_implementation (compositor->xserver_resource,
 				  &xserver_implementation, compositor, NULL);
 
-  wl_resource_post_event (compositor->xserver_resource,
-                          XSERVER_LISTEN_SOCKET,
-                          compositor->xwayland_abstract_fd);
-
-  wl_resource_post_event (compositor->xserver_resource,
-                          XSERVER_LISTEN_SOCKET,
-                          compositor->xwayland_unix_fd);
+  xserver_send_listen_socket (compositor->xserver_resource, compositor->xwayland_abstract_fd);
+  xserver_send_listen_socket (compositor->xserver_resource, compositor->xwayland_unix_fd);
 
   /* Make sure xwayland will recieve the above sockets in a finite
    * time before unblocking the initialization mainloop since we are
@@ -123,7 +120,7 @@ create_lockfile (int display, int *display_out)
               /* ignore error and try the next display number */
               display++;
               continue;
-          }
+            }
           close (fd);
 
           other = strtol (pid, &end, 0);
@@ -135,7 +132,7 @@ create_lockfile (int display, int *display_out)
               /* ignore error and try the next display number */
               display++;
               continue;
-          }
+            }
 
           if (kill (other, 0) < 0 && errno == ESRCH)
             {
@@ -148,7 +145,7 @@ create_lockfile (int display, int *display_out)
                 }
               g_free (filename);
               continue;
-          }
+            }
 
           g_free (filename);
           display++;
@@ -240,11 +237,12 @@ bind_to_unix_socket (int display)
       return -1;
     }
 
-  if (listen (fd, 1) < 0) {
+  if (listen (fd, 1) < 0)
+    {
       unlink (addr.sun_path);
       close (fd);
       return -1;
-  }
+    }
 
   return fd;
 }
@@ -310,8 +308,8 @@ meta_xwayland_start (MetaWaylandCompositor *compositor)
       lockfile = create_lockfile (display, &display);
       if (!lockfile)
         {
-         g_warning ("Failed to create an X lock file");
-         return FALSE;
+          g_warning ("Failed to create an X lock file");
+          return FALSE;
         }
 
       compositor->xwayland_abstract_fd = bind_to_abstract_socket (display);
